//*****************************************************************************
//  SENSOR CONTROLLER STUDIO BENCHMARK: FLOW METER WITH UART LOGGING
//  Operating system: TI-RTOS
//
//
//  Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//****************************************************************************/
#include "scif.h"
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/BIOS.h>
#include <Board.h>
#include <ti/sysbios/family/arm/cc26xx/Power.h>
#include <ti/sysbios/family/arm/cc26xx/PowerCC2650.h>
#include <ti/drivers/UART.h>
#include <ti/drivers/uart/UARTCC26XX.h>
#include "string.h"
#include "stdio.h"
#include "yvals.h"
#include "math.h"
#include <driverlib/aux_adc.h> //for ADC calibration operations



#define BV(n)               (1 << (n))


#define DISABLE_LOW_POWER   0


// Task data
Task_Struct myTask;
Char myTaskStack[1024];


// UART driver objects
UART_Handle uHandle;
UART_Params uParams;
char pLine[25], uart_command;


// Semaphore used to wait for Sensor Controller task ALERT event
static Semaphore_Struct semScTaskAlert;

//ADC calculation objects
uint32_t calADC12_gain=0, lmt70_HeatSensor=0, lmt70_RoomSensor=0;
int8_t calADC12_offset=0, testmode=0;
float temp1_LMT70lin,temp2_LMT70lin, temp1_LMT70dual,temp2_LMT70dual;
float a = -7.857923E-06, b = -1.777501E-01, c = 2.046398E+02;


void scCtrlReadyCallback(void) {

} // scCtrlReadyCallback


void scTaskAlertCallback(void) {

    // Wake up the OS task
    Semaphore_post(Semaphore_handle(&semScTaskAlert));

} // scTaskAlertCallback


void taskFxn(UArg a0, UArg a1) {

    // Initialize UART parameters
    UART_Params_init(&uParams);
    //uParams.baudRate      = 57600;
    uParams.baudRate      = 9600;
    uParams.writeDataMode = UART_DATA_BINARY; //must be TEXT and NOT "BINARY"
    uParams.readDataMode = UART_DATA_BINARY; //MS 1. Oct'15
    uParams.readReturnMode = UART_RETURN_FULL; //MS
    uParams.dataLength    = UART_LEN_8;
    uParams.stopBits      = UART_STOP_ONE;
    uParams.readMode    =  UART_MODE_BLOCKING; //UART_MODE_CALLBACK; //    UART_MODE_BLOCKING

    // Open UART
    uHandle = UART_open(Board_UART, &uParams);

    // Initialize the Sensor Controller
    scifOsalInit();
    scifOsalRegisterCtrlReadyCallback(scCtrlReadyCallback);
    scifOsalRegisterTaskAlertCallback(scTaskAlertCallback);
    scifInit(&scifDriverSetup);

    // Set the Sensor Controller task tick interval to 1/16 second
     scifStartRtcTicksNow(0x00010000 * 1); //was /16; now 1 per every second

    // Start the Sensor Controller task (not to be confused with OS tasks)
    scifStartTasksNbl(BV(SCIF_LMT70POWER_READ_TASK_ID));

     // Read out ADC calibration values for Analog Output from LMT70 to CC1310 ADC
    calADC12_offset = AUXADCGetAdjustmentOffset(AUXADC_REF_FIXED);
    calADC12_gain = AUXADCGetAdjustmentGain(AUXADC_REF_FIXED);
    
    
    // Main loop
    while (1) {
      
        // Wait for an ALERT callback
        Semaphore_pend(Semaphore_handle(&semScTaskAlert), BIOS_WAIT_FOREVER);

        // Clear the ALERT interrupt source
        scifClearAlertIntSource();

        // Print ADC values = Analog Output from LMT70 to CC1310 ADC
        calADC12_offset = AUXADCGetAdjustmentOffset(AUXADC_REF_FIXED);
        calADC12_gain = AUXADCGetAdjustmentGain(AUXADC_REF_FIXED);
        
        lmt70_HeatSensor = AUXADCAdjustValueForGainAndOffset(scifTaskData.lmt70powerRead.output.adcVal1, calADC12_gain, calADC12_offset);
        lmt70_RoomSensor = AUXADCAdjustValueForGainAndOffset(scifTaskData.lmt70powerRead.output.adcVal2, calADC12_gain, calADC12_offset);
        
        //Linear Interpolation method for LMT70 temp. calculation
        // Thus the final equation used to calculate the measured temperature (TM) in the range between 20C and 30C is: TM = m  VTAO + b 
        // TM = 0.193 C/mV  VTAO + 212.009C
        // where VTAO is in mV and TM is in C.
        temp1_LMT70lin = -0.193 * lmt70_HeatSensor * 1440 / 4095 + 212.009;
        temp2_LMT70lin = -0.193 * lmt70_RoomSensor * 1440 / 4095 + 212.009;
        
      
        temp1_LMT70dual = a * lmt70_HeatSensor *  lmt70_HeatSensor * 4300 * 4300 * 1408/4095 * 1408/4095/4095/4095 + b * lmt70_HeatSensor * 4300 * 1408/(4095*4095) + c;
        temp2_LMT70dual = a * lmt70_RoomSensor *  lmt70_RoomSensor * 4300 * 4300 * 1408/4095 * 1408/4095/4095/4095 + b * lmt70_RoomSensor * 4300 * 1408/(4095*4095) + c;
   
        
        // TM = a (VTAO)2+ b (VTAO) + c  if Second Order Transfer Function is used (instead of Linear)
        // a = -7.857923E-06; b = -1.777501E-01; c = 2.046398E+02
                
   //   sprintf(pLine + 0,  "%4d ",  scifTaskData.lmt70powerRead.output.adcVal1);  //use 4d for Decimal - ADC read out value
   //   sprintf(pLine + 5,  "%4d\n",  scifTaskData.lmt70powerRead.output.adcVal2);      //use 4d for Decimal - ADC read out value; 23.9.15
        
        
        sprintf(pLine + 0,  "%4d",  lmt70_HeatSensor);  //use 4d for Decimal
        sprintf(pLine + 4,  ",");  //use , for Separator
        
        sprintf(pLine + 5,  "%4d",  lmt70_RoomSensor);      //use 4d for Decimal 23.9.15
        sprintf(pLine + 9,  ",");  //use , for Separator
        
        sprintf(pLine + 10, "%2.2f",  temp1_LMT70dual); //5 digits incl. dec. point
        sprintf(pLine + 15,  ",");      //use , for Separator
        sprintf(pLine + 16, "%2.2f\r\n", temp2_LMT70dual);
        
        if (testmode == 0) 
          UART_write(uHandle, pLine, strlen(pLine));
        
        //wait for Input over UART RX       
       UART_read(uHandle, &uart_command, 1); //UART_read
        
        switch (uart_command) {
          case 'g': //STOP command
         // Send it over the UART
            testmode = 1;
            break;
          case 's': //start
           testmode = 0;
            break;
          default:
            break;
        }  
        
        // Acknowledge the alert event
        scifAckAlertEvents();
    }
   
} // taskFxn


int main(void) {
    Task_Params taskParams;

    // Optional: Prevent the system from entering standby to allow debugging
#if DISABLE_LOW_POWER
    Power_setConstraint(Power_SB_DISALLOW);
    Power_setConstraint(Power_IDLE_PD_DISALLOW);
#endif

    // Initialize the PIN driver
    PIN_init(BoardGpioInitTable);

    // Configure the OS task
    Task_Params_init(&taskParams);
    taskParams.stack = myTaskStack;
    taskParams.stackSize = sizeof(myTaskStack);
    taskParams.priority = 3;
    Task_construct(&myTask, taskFxn, &taskParams, NULL);

    // Create the semaphore used to wait for Sensor Controller ALERT events
    Semaphore_Params semParams;
    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    Semaphore_construct(&semScTaskAlert, 0, &semParams);

    // Start TI-RTOS
    BIOS_start();
    return 0;

} // main
